@@ -1,16 +1,19 @@
-#include <filezilla.h>
 #include "controlsocket.h"
 #include "directorycache.h"
 #include "engineprivate.h"
+#include "filezilla.h"
 
-CFileZillaEngine::CFileZillaEngine(CFileZillaEngineContext& engine_context, EngineNotificationHandler& notificationHandler)
-	: impl_(new CFileZillaEnginePrivate(engine_context, *this, notificationHandler))
+CFileZillaEngine::CFileZillaEngine(CFileZillaEngineContext& engine_context, std::function<void(CFileZillaEngine*)> const& cb)
+	: impl_(std::make_unique<CFileZillaEnginePrivate>(engine_context, *this, cb))
 {
 }
 
 CFileZillaEngine::~CFileZillaEngine()
 {
-	delete impl_;
+	if (impl_) {
+		impl_->shutdown();
+		impl_.reset();
+	}
 }
 
 int CFileZillaEngine::Execute(const CCommand &command)
@@ -33,11 +36,6 @@ bool CFileZillaEngine::IsPendingAsyncRequestReply(std::unique_ptr<CAsyncRequestN
 	return impl_->IsPendingAsyncRequestReply(pNotification);
 }
 
-bool CFileZillaEngine::IsActive(CFileZillaEngine::_direction direction)
-{
-	return CFileZillaEnginePrivate::IsActive(direction);
-}
-
 CTransferStatus CFileZillaEngine::GetTransferStatus(bool &changed)
 {
 	return impl_->GetTransferStatus(changed);
