@@ -10,7 +10,7 @@
 
 #include "../include/local_path.h"
 #include "../include/engine_options.h"
-#include "../include/sizeformatting_base.h"
+#include "../include/sizeformatting.h"
 
 #include <libfilezilla/event_loop.hpp>
 #include <libfilezilla/iputils.hpp>
@@ -79,7 +79,7 @@ void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpDat
 		std::wstring time = fz::sprintf(fztranslate("%d second", "%d seconds", elapsed), elapsed);
 
 		int64_t transferred = status.currentOffset - status.startOffset;
-		std::wstring size = CSizeFormatBase::Format(&engine_.GetOptions(), transferred, true);
+		std::wstring size = engine_.GetContext().size_formatter().Format(transferred, SizeFormatPurpose::in_line);
 
 		logmsg::type msgType = logmsg::error;
 		std::wstring msg;
