@@ -60,7 +60,7 @@ CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& contex
 		m_engineList.push_back(this);
 	}
 
-	logger_ = std::make_unique<CLogging>(*this);
+	logger_ = std::make_unique<CLogging>(*this, context_.GetLogFileWriter());
 
 	{
 		bool queue_logs = ShouldQueueLogsFromOptions();
@@ -353,7 +353,9 @@ int CFileZillaEnginePrivate::List(CListCommand const& command)
 						server.GetProtocol() == SWIFT || server.GetProtocol() == GOOGLE_CLOUD ||
 						server.GetProtocol() == GOOGLE_DRIVE || server.GetProtocol() == DROPBOX ||
 						server.GetProtocol() == ONEDRIVE || server.GetProtocol() == B2 ||
-						server.GetProtocol() == BOX || server.GetProtocol() == RACKSPACE || server.GetProtocol() == STORJ_GRANT) {
+						server.GetProtocol() == BOX || server.GetProtocol() == RACKSPACE ||
+						server.GetProtocol() == STORJ_GRANT || server.GetProtocol() == GOOGLE_CLOUD_SVC_ACC ||
+						server.GetProtocol() == S3_SSO || server.GetProtocol() == CLOUDFLARE_R2) {
 					path = command.GetPath();
 					path.ChangePath(command.GetSubDir());
 				}
