@@ -6,15 +6,6 @@
 
 class CDirectoryListingParser;
 
-enum listStates
-{
-	list_init,
-	list_waitcwd,
-	list_waitlock,
-	list_waittransfer,
-	list_mdtm
-};
-
 class CFtpListOpData final : public COpData, public CFtpOpData, public CFtpTransferOpData
 {
 public:
