@@ -1,84 +1,65 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "chmod.h"
 #include "connect.h"
 #include "cwd.h"
 #include "delete.h"
-#include "../directorycache.h"
-#include "directorylistingparser.h"
-#include "engineprivate.h"
 #include "event.h"
 #include "filetransfer.h"
 #include "list.h"
-#include "input_thread.h"
+#include "input_parser.h"
 #include "mkd.h"
-#include "pathcache.h"
-#include "proxy.h"
 #include "rename.h"
 #include "rmd.h"
-#include "servercapabilities.h"
 #include "sftpcontrolsocket.h"
 
+#include "../directorycache.h"
+#include "../directorylistingparser.h"
+#include "../engineprivate.h"
+#include "../pathcache.h"
+#include "../proxy.h"
+#include "../servercapabilities.h"
+
+#include "../../include/engine_options.h"
+
 #include <libfilezilla/event_loop.hpp>
-#include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/process.hpp>
 
 #include <algorithm>
-#include <cwchar>
+
+#include <assert.h>
+
+#ifndef FZ_WINDOWS
+#include <unistd.h>
+#endif
+
+struct SftpRateAvailableEventType;
+typedef fz::simple_event<SftpRateAvailableEventType, fz::direction::type> SftpRateAvailableEvent;
 
 CSftpControlSocket::CSftpControlSocket(CFileZillaEnginePrivate & engine)
-	: CControlSocket(engine)
+	: CControlSocket(engine, true)
 {
 	m_useUTF8 = true;
 }
 
 CSftpControlSocket::~CSftpControlSocket()
 {
+	remove_bucket();
 	remove_handler();
 	DoClose();
 }
 
 void CSftpControlSocket::Connect(CServer const& server, Credentials const& credentials)
 {
-	log(logmsg::status, _("Connecting to %s..."), server.Format(ServerFormat::with_optional_port, credentials));
-	SetWait(true);
-
-	m_sftpEncryptionDetails = CSftpEncryptionNotification();
-
 	if (server.GetEncodingType() == ENCODING_CUSTOM) {
 		log(logmsg::debug_info, L"Using custom encoding: %s", server.GetCustomEncoding());
 		m_useUTF8 = false;
 	}
-	else {
-		m_useUTF8 = true;
-	}
 
 	currentServer_ = server;
+	credentials_ = credentials;
 
-	auto pData = std::make_unique<CSftpConnectOpData>(*this, credentials);
-	if (!credentials.keyFile_.empty()) {
-		pData->keyfiles_ = fz::strtok(credentials.keyFile_, L"\r\n");
-	}
-	else {
-		pData->keyfiles_ = fz::strtok(engine_.GetOptions().GetOption(OPTION_SFTP_KEYFILES), L"\r\n");
-	}
-
-	pData->keyfiles_.erase(
-		std::remove_if(pData->keyfiles_.begin(), pData->keyfiles_.end(),
-			[this](std::wstring const& keyfile) {
-				if (fz::local_filesys::get_file_type(fz::to_native(keyfile), true) != fz::local_filesys::file) {
-					log(logmsg::status, _("Skipping non-existing key file \"%s\""), keyfile);
-					return true;
-				}
-				return false;
-		}), pData->keyfiles_.end());
-
-	pData->keyfile_ = pData->keyfiles_.cbegin();
-
-	process_ = std::make_unique<fz::process>();
-
-	engine_.GetRateLimiter().AddObject(this);
-	Push(std::move(pData));
+	Push(std::make_unique<CSftpConnectOpData>(*this));
 }
 
 void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
@@ -87,7 +68,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		return;
 	}
 
-	if (!input_thread_) {
+	if (!input_parser_) {
 		return;
 	}
 
@@ -125,10 +106,10 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		log_raw(logmsg::status, message.text[0]);
 		break;
 	case sftpEvent::Recv:
-		SetActive(CFileZillaEngine::recv);
+		RecordActivity(activity_logger::recv, fz::to_integral<uint64_t>(message.text[0]));
 		break;
 	case sftpEvent::Send:
-		SetActive(CFileZillaEngine::send);
+		RecordActivity(activity_logger::send, fz::to_integral<uint64_t>(message.text[0]));
 		break;
 	case sftpEvent::Transfer:
 		{
@@ -136,23 +117,26 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 
 			bool tmp;
 			CTransferStatus status = engine_.transfer_status_.Get(tmp);
-			if (!status.empty() && !status.madeProgress) {
+			if (!status.empty()) {
 				if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
 					auto & data = static_cast<CSftpFileTransferOpData &>(*operations_.back());
-					if (data.download_) {
-						if (value > 0) {
-							engine_.transfer_status_.SetMadeProgress();
-						}
+					if (!data.download()) {
+						engine_.transfer_status_.Update(value);	
 					}
-					else {
-						if (status.currentOffset > status.startOffset + 65565) {
-							engine_.transfer_status_.SetMadeProgress();
+					if (!status.madeProgress) {
+						if (data.download()) {
+							if (value > 0) {
+								engine_.transfer_status_.SetMadeProgress();
+							}
+						}
+						else {
+							if (status.currentOffset > status.startOffset + 65565) {
+								engine_.transfer_status_.SetMadeProgress();
+							}
 						}
 					}
 				}
 			}
-
-			engine_.transfer_status_.Update(value);
 		}
 		break;
 	case sftpEvent::AskHostkey:
@@ -163,7 +147,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 				DoClose(FZ_REPLY_INTERNALERROR);
 				break;
 			}
-			SendAsyncRequest(new CHostKeyNotification(message.text[0], port, m_sftpEncryptionDetails, message.type == sftpEvent::AskHostkeyChanged));
+			SendAsyncRequest(std::make_unique<CHostKeyNotification>(message.text[0], port, m_sftpEncryptionDetails, message.type == sftpEvent::AskHostkeyChanged));
 		}
 		break;
 	case sftpEvent::AskHostkeyBetteralg:
@@ -181,7 +165,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 			std::wstring const challengeIdentifier = m_requestPreamble + L"\n" + m_requestInstruction + L"\n" + message.text[0];
 
 			CInteractiveLoginNotification::type t = CInteractiveLoginNotification::interactive;
-			if (data.credentials_.logonType_ == LogonType::interactive || m_requestPreamble == L"SSH key passphrase") {
+			if (credentials_.logonType_ == LogonType::interactive || m_requestPreamble == L"SSH key passphrase") {
 				if (m_requestPreamble == L"SSH key passphrase") {
 					t = CInteractiveLoginNotification::keyfile;
 				}
@@ -196,12 +180,12 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 				if (message.text[0] != L"Password:") {
 					challenge += message.text[0];
 				}
-				CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(t, challenge, data.lastChallenge == challengeIdentifier);
-				pNotification->server = currentServer_;
-				pNotification->handle_ = handle_;
-				pNotification->credentials = data.credentials_;
+				auto notification = std::make_unique<CInteractiveLoginNotification>(t, challenge, data.lastChallenge == challengeIdentifier);
+				notification->server = currentServer_;
+				notification->handle_ = handle_;
+				notification->credentials = credentials_;
 
-				SendAsyncRequest(pNotification);
+				SendAsyncRequest(std::move(notification));
 			}
 			else {
 				if (!data.lastChallenge.empty() && data.lastChallengeType != CInteractiveLoginNotification::keyfile) {
@@ -216,7 +200,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 					return;
 				}
 
-				std::wstring const pass = (data.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : data.credentials_.GetPass();
+				std::wstring const pass = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : credentials_.GetPass();
 				std::wstring show = L"Pass: ";
 				show.append(pass.size(), '*');
 				SendCommand(pass, show);
@@ -232,10 +216,10 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		m_requestInstruction = message.text[0];
 		break;
 	case sftpEvent::UsedQuotaRecv:
-		OnQuotaRequest(CRateLimiter::inbound);
+		OnQuotaRequest(fz::direction::inbound);
 		break;
 	case sftpEvent::UsedQuotaSend:
-		OnQuotaRequest(CRateLimiter::outbound);
+		OnQuotaRequest(fz::direction::outbound);
 		break;
 	case sftpEvent::KexAlgorithm:
 		m_sftpEncryptionDetails.kexAlgorithm = message.text[0];
@@ -260,13 +244,9 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		break;
 	case sftpEvent::Hostkey:
 		{
-			auto tokens = fz::strtok(message.text[0], ' ');
+			auto tokens = fz::strtok_view(message.text[0], ' ');
 			if (!tokens.empty()) {
-				m_sftpEncryptionDetails.hostKeyFingerprintSHA256 = tokens.back();
-				tokens.pop_back();
-			}
-			if (!tokens.empty()) {
-				m_sftpEncryptionDetails.hostKeyFingerprintMD5 = tokens.back();
+				m_sftpEncryptionDetails.hostKeyFingerprint = tokens.back();
 				tokens.pop_back();
 			}
 			for (auto const& token : tokens) {
@@ -277,6 +257,30 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 			}
 		}
 		break;
+	case sftpEvent::io_nextbuf:
+		if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
+			auto & data = static_cast<CSftpFileTransferOpData&>(*operations_.back());
+			data.OnNextBufferRequested(fz::to_integral<uint64_t>(message.text[0]));
+		}
+		break;
+	case sftpEvent::io_open:
+		if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
+			auto & data = static_cast<CSftpFileTransferOpData&>(*operations_.back());
+			data.OnOpenRequested(fz::to_integral<uint64_t>(message.text[0]));
+		}
+		break;
+	case sftpEvent::io_size:
+		if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
+			auto & data = static_cast<CSftpFileTransferOpData&>(*operations_.back());
+			data.OnSizeRequested();
+		}
+		break;
+	case sftpEvent::io_finalize:
+		if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
+			auto & data = static_cast<CSftpFileTransferOpData&>(*operations_.back());
+			data.OnFinalizeRequested(fz::to_integral<uint64_t>(message.text[0]));
+		}
+		break;
 	default:
 		log(logmsg::debug_warning, L"Message type %d not handled", message.type);
 		break;
@@ -289,7 +293,7 @@ void CSftpControlSocket::OnSftpListEvent(sftp_list_message const& message)
 		return;
 	}
 
-	if (!input_thread_) {
+	if (!input_parser_) {
 		return;
 	}
 
@@ -305,19 +309,6 @@ void CSftpControlSocket::OnSftpListEvent(sftp_list_message const& message)
 	}
 }
 
-void CSftpControlSocket::OnTerminate(std::wstring const& error)
-{
-	if (!error.empty()) {
-		log_raw(logmsg::error, error);
-	}
-	else {
-		log_raw(logmsg::debug_info, L"CSftpControlSocket::OnTerminate without error");
-	}
-	if (process_) {
-		DoClose();
-	}
-}
-
 int CSftpControlSocket::SendCommand(std::wstring const& cmd, std::wstring const& show)
 {
 	SetWait(true);
@@ -333,10 +324,10 @@ int CSftpControlSocket::SendCommand(std::wstring const& cmd, std::wstring const&
 		return FZ_REPLY_INTERNALERROR;
 	}
 
-	return AddToStream(cmd + L"\n");
+	return AddToSendBuffer(cmd + L"\n");
 }
 
-int CSftpControlSocket::AddToStream(std::wstring const& cmd)
+int CSftpControlSocket::AddToSendBuffer(std::wstring const& cmd)
 {
 	std::string const str = ConvToServer(cmd);
 	if (str.empty()) {
@@ -344,17 +335,25 @@ int CSftpControlSocket::AddToStream(std::wstring const& cmd)
 		return FZ_REPLY_ERROR;
 	}
 
-	return AddToStream(str);
+	return AddToSendBuffer(str);
 }
 
-int CSftpControlSocket::AddToStream(std::string const& cmd)
+int CSftpControlSocket::AddToSendBuffer(std::string const& cmd)
 {
 	if (!process_) {
 		return FZ_REPLY_INTERNALERROR;
 	}
 
-	if (!process_->write(cmd)) {
-		return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
+
+	bool const can_send = send_buffer_.empty();
+
+	send_buffer_.append(cmd);
+
+	if (can_send) {
+		return SendToProcess();
+	}
+	else {
+		return FZ_REPLY_WOULDBLOCK;
 	}
 
 	return FZ_REPLY_WOULDBLOCK;
@@ -413,8 +412,6 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 				return false;
 			}
 
-			auto & data = static_cast<CSftpConnectOpData&>(*operations_.back());
-
 			auto *pInteractiveLoginNotification = static_cast<CInteractiveLoginNotification *>(pNotification);
 
 			if (!pInteractiveLoginNotification->passwordSet) {
@@ -423,7 +420,7 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 			}
 			std::wstring const& pass = pInteractiveLoginNotification->credentials.GetPass();
 			if (pInteractiveLoginNotification->GetType() != CInteractiveLoginNotification::keyfile) {
-				data.credentials_.SetPass(pass);
+				credentials_.SetPass(pass);
 			}
 			std::wstring show = L"Pass: ";
 			show.append(pass.size(), '*');
@@ -440,21 +437,6 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 
 void CSftpControlSocket::List(CServerPath const& path, std::wstring const& subDir, int flags)
 {
-	CServerPath newPath = currentPath_;
-	if (!path.empty()) {
-		newPath = path;
-	}
-	if (!newPath.ChangePath(subDir)) {
-		newPath.clear();
-	}
-
-	if (newPath.empty()) {
-		log(logmsg::status, _("Retrieving directory listing..."));
-	}
-	else {
-		log(logmsg::status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
-	}
-
 	Push(std::make_unique<CSftpListOpData>(*this, path, subDir, flags));
 }
 
@@ -466,7 +448,7 @@ void CSftpControlSocket::ChangeDir(CServerPath const& path, std::wstring const&
 	pData->link_discovery_ = link_discovery;
 
 	if (!operations_.empty() && operations_.back()->opId == Command::transfer &&
-		!static_cast<CSftpFileTransferOpData&>(*operations_.back()).download_)
+		!static_cast<CSftpFileTransferOpData&>(*operations_.back()).download())
 	{
 		pData->tryMkdOnFail_ = true;
 		assert(subDir.empty());
@@ -478,13 +460,21 @@ void CSftpControlSocket::ChangeDir(CServerPath const& path, std::wstring const&
 void CSftpControlSocket::ProcessReply(int result, std::wstring const& reply)
 {
 	result_ = result;
-	response_ = reply;
+	response_.clear();
 
 	if (operations_.empty()) {
 		log(logmsg::debug_info, L"Skipping reply without active operation.");
 		return;
 	}
 
+	if (reply.size() > 65536) {
+		log(fz::logmsg::error, _("Received too long response line, closing connection."));
+		DoClose(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
+		return;
+	}
+
+	response_ = reply;
+
 	auto & data = *operations_.back();
 	log(logmsg::debug_verbose, L"%s::ParseResponse() in state %d", data.name_, data.opState);
 	int res = data.ParseResponse();
@@ -506,38 +496,34 @@ void CSftpControlSocket::ProcessReply(int result, std::wstring const& reply)
 		}
 	}
 }
-void CSftpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
-									std::wstring const& remoteFile, bool download,
-									CFileTransferCommand::t_transferSettings const& transferSettings)
+void CSftpControlSocket::FileTransfer(CFileTransferCommand const& cmd)
 {
-	auto pData = std::make_unique<CSftpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath, transferSettings);
-	Push(std::move(pData));
+	Push(std::make_unique<CSftpFileTransferOpData>(*this, cmd));
 }
 
 int CSftpControlSocket::DoClose(int nErrorCode)
 {
-	engine_.GetRateLimiter().RemoveObject(this);
-
+	remove_bucket();
 	if (process_) {
 		process_->kill();
 	}
 
-	if (input_thread_) {
-		input_thread_.reset();
+	if (input_parser_) {
+		input_parser_.reset();
 
-		auto threadEventsFilter = [&](fz::event_loop::Events::value_type const& ev) -> bool {
-			if (ev.first != this) {
-				return false;
-			}
-			else if (ev.second->derived_type() == CSftpEvent::type() || ev.second->derived_type() == CTerminateEvent::type()) {
+		auto threadEventsFilter = [](fz::event_base const& ev) -> bool {
+			if (ev.derived_type() == CSftpEvent::type() || ev.derived_type() == CSftpListEvent::type()) {
 				return true;
 			}
 			return false;
 		};
 
-		event_loop_.filter_events(threadEventsFilter);
+		filter_events(threadEventsFilter);
 	}
 	process_.reset();
+
+	m_sftpEncryptionDetails = CSftpEncryptionNotification();
+
 	return CControlSocket::DoClose(nErrorCode);
 }
 
@@ -548,17 +534,8 @@ void CSftpControlSocket::Cancel()
 	}
 }
 
-void CSftpControlSocket::Mkdir(CServerPath const& path)
+void CSftpControlSocket::Mkdir(CServerPath const& path, transfer_flags const&)
 {
-	/* Directory creation works like this: First find a parent directory into
-	 * which we can CWD, then create the subdirs one by one. If either part
-	 * fails, try MKD with the full path directly.
-	 */
-
-	if (operations_.empty()) {
-		log(logmsg::status, _("Creating directory '%s'..."), path.GetPath());
-	}
-
 	auto pData = std::make_unique<CSftpMkdirOpData>(*this);
 	pData->path_ = path;
 	Push(std::move(pData));
@@ -575,7 +552,7 @@ void CSftpControlSocket::Delete(const CServerPath& path, std::vector<std::wstrin
 	assert(!files.empty());
 
 	log(logmsg::debug_verbose, L"CSftpControlSocket::Delete");
-	
+
 	auto pData = std::make_unique<CSftpDeleteOpData>(*this);
 	pData->path_ = path;
 	pData->files_ = std::move(files);
@@ -594,76 +571,100 @@ void CSftpControlSocket::RemoveDir(CServerPath const& path, std::wstring const&
 
 void CSftpControlSocket::Chmod(CChmodCommand const& command)
 {
-	log(logmsg::status, _("Setting permissions of '%s' to '%s'"), command.GetPath().FormatFilename(command.GetFile()), command.GetPermission());
 	Push(std::make_unique<CSftpChmodOpData>(*this, command));
 }
 
 void CSftpControlSocket::Rename(CRenameCommand const& command)
 {
-	log(logmsg::status, _("Renaming '%s' to '%s'"), command.GetFromPath().FormatFilename(command.GetFromFile()), command.GetToPath().FormatFilename(command.GetToFile()));
 	Push(std::make_unique<CSftpRenameOpData>(*this, command));
 }
 
-std::wstring CSftpControlSocket::WildcardEscape(std::wstring const& file)
+void CSftpControlSocket::wakeup(fz::direction::type const d)
 {
-	std::wstring ret;
-	// see src/putty/wildcard.c
+	send_event<SftpRateAvailableEvent>(d);
+}
 
-	ret.reserve(file.size());
-	for (size_t i = 0; i < file.size(); ++i) {
-		auto const& c = file[i];
-		switch (c)
-		{
-		case '[':
-		case ']':
-		case '*':
-		case '?':
-		case '\\':
-			ret.push_back('\\');
-			break;
-		default:
-			break;
-		}
-		ret.push_back(c);
+void CSftpControlSocket::OnQuotaRequest(fz::direction::type const d)
+{
+	if (!process_) {
+		return;
 	}
-	return ret;
-}
 
-void CSftpControlSocket::OnRateAvailable(CRateLimiter::rate_direction direction)
-{
-	OnQuotaRequest(direction);
-}
-
-void CSftpControlSocket::OnQuotaRequest(CRateLimiter::rate_direction direction)
-{
-	int64_t bytes = GetAvailableBytes(direction);
-	if (bytes > 0) {
+	fz::rate::type bytes = available(d);
+	if (bytes == fz::rate::unlimited) {
+		AddToSendBuffer(fz::sprintf("-%d-\n", d));
+	}
+	else if (bytes > 0) {
 		int b;
-		if (bytes > std::numeric_limits<int>::max()) {
+		if (bytes > static_cast<size_t>(std::numeric_limits<int>::max())) {
 			b = std::numeric_limits<int>::max();
 		}
 		else {
 			b = static_cast<int>(bytes);
 		}
-		AddToStream(fz::sprintf("-%d%d,%d\n", direction, b, engine_.GetOptions().GetOptionVal(OPTION_SPEEDLIMIT_INBOUND + static_cast<int>(direction))));
-		UpdateUsage(direction, b);
-	}
-	else if (bytes == 0) {
-		Wait(direction);
-	}
-	else if (bytes < 0) {
-		AddToStream(fz::sprintf("-%d-\n", direction));
+		AddToSendBuffer(fz::sprintf("-%d%d,%d\n", d, b, engine_.GetOptions().get_int(d ? OPTION_SPEEDLIMIT_OUTBOUND : OPTION_SPEEDLIMIT_INBOUND)));
+		consume(d, static_cast<fz::rate::type>(b));
 	}
 }
 
 void CSftpControlSocket::operator()(fz::event_base const& ev)
 {
-	if (fz::dispatch<CSftpEvent, CSftpListEvent, CTerminateEvent>(ev, this,
+	if (fz::dispatch<fz::process_event, CSftpEvent, CSftpListEvent, SftpRateAvailableEvent>(ev, this,
+		&CSftpControlSocket::OnProcessEvent,
 		&CSftpControlSocket::OnSftpEvent,
 		&CSftpControlSocket::OnSftpListEvent,
-		&CSftpControlSocket::OnTerminate)) {
+		&CSftpControlSocket::OnQuotaRequest)) {
 		return;
 	}
 
 	CControlSocket::operator()(ev);
 }
+
+void CSftpControlSocket::Push(std::unique_ptr<COpData> && pNewOpData)
+{
+	CControlSocket::Push(std::move(pNewOpData));
+	if (operations_.size() == 1 && operations_.back()->opId != Command::connect) {
+		if (!process_) {
+			std::unique_ptr<COpData> connOp = std::make_unique<CSftpConnectOpData>(*this);
+			connOp->topLevelOperation_ = true;
+			CControlSocket::Push(std::move(connOp));
+		}
+	}
+}
+
+int CSftpControlSocket::SendToProcess()
+{
+	if (!process_) {
+		return FZ_REPLY_INTERNALERROR;
+	}
+	while (!send_buffer_.empty()) {
+		fz::rwresult res = process_->write(send_buffer_.get(), send_buffer_.size());
+		if (res) {
+			send_buffer_.consume(res.value_);
+		}
+		else if (res.error_ == fz::rwresult::wouldblock) {
+			break;
+		}
+		else {
+			log(logmsg::error, _("Could not send command to fzsftp executable"));
+			return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
+		}
+	}
+	return FZ_REPLY_WOULDBLOCK;
+}
+
+void CSftpControlSocket::OnProcessEvent(fz::process*, fz::process_event_flag const& f)
+{
+	if (f == fz::process_event_flag::write) {
+		int res = SendToProcess();
+		if (res != FZ_REPLY_WOULDBLOCK) {
+			DoClose(res);
+		}
+	}
+	else {
+		int res = input_parser_->OnData();
+		if (res != FZ_REPLY_WOULDBLOCK) {
+			DoClose(res);
+		}
+	}
+}
