@@ -8,11 +8,7 @@
   #define FZC_PUBLIC_SYMBOL FZ_EXPORT_PUBLIC
   #define FZC_PRIVATE_SYMBOL FZ_EXPORT_PRIVATE
 #else
-  #if FZC_USING_DLL
-    #define FZC_PUBLIC_SYMBOL FZ_IMPORT_SHARED
-  #else
-    #define FZC_PUBLIC_SYMBOL FZ_IMPORT_STATIC
-  #endif
+  #define FZC_PUBLIC_SYMBOL FZ_IMPORT_SHARED
   #define FZC_PRIVATE_SYMBOL
 #endif
 
