@@ -18,10 +18,12 @@
 #include "queue.h"
 #include "RemoteListView.h"
 #include "remote_recursive_operation.h"
-#include "sizeformatting.h"
 #include "timeformatting.h"
+
 #include "../commonui/misc.h"
 
+#include "../include/sizeformatting.h"
+
 #include <wx/clipbrd.h>
 #include <wx/menu.h>
 
@@ -314,10 +316,11 @@ BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CRemoteListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CRemoteListView::CRemoteListView(CView* pParent, CState& state, CQueueView* pQueue, COptionsBase & options)
+CRemoteListView::CRemoteListView(CView* pParent, CState& state, CQueueView* pQueue, COptionsBase & options, CEditHandler* edit_handler)
 	: CFileListCtrl<CGenericFileData>(pParent, pQueue, options)
 	, CStateEventHandler(state)
 	, m_parentView(pParent)
+	, edit_handler_(edit_handler)
 {
 	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
@@ -2353,14 +2356,13 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent&)
 		selected_items.push_back({entry.name, entry.size});
 	}
 
-	CEditHandler* pEditHandler = CEditHandler::Get();
-	if (!pEditHandler) {
+	if (!edit_handler_) {
 		wxBell();
 		return;
 	}
 
 	CServerPath const path = m_pDirectoryListing->path;
-	pEditHandler->Edit(CEditHandler::remote, selected_items, path, m_state.GetSite(), this);
+	edit_handler_->Edit(CEditHandler::remote, selected_items, path, m_state.GetSite(), this);
 }
 
 #ifdef __WXDEBUG__
@@ -2527,7 +2529,7 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 			return wxString();
 		}
 		else {
-			return CSizeFormat::Format(entry.size);
+			return SizeFormatter(options_).Format(entry.size, SizeFormatPurpose::normal);
 		}
 	}
 	else if (column == 2) {
@@ -2791,10 +2793,13 @@ void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 		}
 	}
 
-	CEditHandler* edithandler = CEditHandler::Get(); // Used to get the temporary folder
+	if (!edit_handler_) {
+		wxBell();
+		return;
+	}
 
 	std::wstring const emptyfile_name = L"empty_file_yq744zm";
-	std::wstring emptyfile = edithandler->GetLocalDirectory() + emptyfile_name;
+	std::wstring emptyfile = edit_handler_->GetLocalDirectory() + emptyfile_name;
 
 	// Create the empty temporary file and update its modification time
 	{
