@@ -4,10 +4,11 @@
 #include "queue.h"
 #include "queueview_failed.h"
 #include "queueview_successful.h"
-#include "sizeformatting.h"
 #include "timeformatting.h"
 #include "themeprovider.h"
 
+#include "../include/sizeformatting.h"
+
 #include <wx/filedlg.h>
 
 CQueueItem::CQueueItem(CQueueItem* parent)
@@ -1031,7 +1032,7 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 				{
 					auto const& size = pFileItem->GetSize();
 					if (size >= 0) {
-						return CSizeFormat::Format(size);
+						return SizeFormatter(options_).Format(size, SizeFormatPurpose::normal);
 					}
 					else {
 						return _T("?");
@@ -1641,9 +1642,9 @@ CQueue::CQueue(wxWindow* parent, CMainFrame *pMainFrame, CAsyncRequestQueue *pAs
 	m_pQueueView = new CQueueView(this, 0, pMainFrame, pAsyncRequestQueue, certStore);
 	AddPage(m_pQueueView, m_pQueueView->GetTitle());
 
-	m_pQueueView_Failed = new CQueueViewFailed(this, pMainFrame->GetOptions(), 1);
+	m_pQueueView_Failed = new CQueueViewFailed(this, pMainFrame->GetOptions(), 1, pMainFrame);
 	AddPage(m_pQueueView_Failed, m_pQueueView_Failed->GetTitle());
-	m_pQueueView_Successful = new CQueueViewSuccessful(this, pMainFrame->GetOptions(), 2);
+	m_pQueueView_Successful = new CQueueViewSuccessful(this, pMainFrame->GetOptions(), 2, pMainFrame);
 	AddPage(m_pQueueView_Successful, m_pQueueView_Successful->GetTitle());
 
 	RemoveExtraBorders();
