@@ -159,6 +159,7 @@ public:
 	CServerPath const& GetRemotePath(int64_t id) const;
 
 	bool Bind(sqlite3_stmt* statement, int index, int value);
+	bool Bind(sqlite3_stmt* statement, int index, unsigned int value);
 	bool Bind(sqlite3_stmt* statement, int index, int64_t value);
 	bool Bind(sqlite3_stmt* statement, int index, std::wstring const& value);
 	bool Bind(sqlite3_stmt* statement, int index, std::string const& value);
@@ -619,6 +620,11 @@ bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, int value)
 	return sqlite3_bind_int(statement, index, value) == SQLITE_OK;
 }
 
+bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, unsigned int value)
+{
+	return sqlite3_bind_int64(statement, index, value) == SQLITE_OK;
+}
+
 
 bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, int64_t value)
 {
@@ -743,7 +749,7 @@ bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 		Bind(insertServerQuery_, server_table_column_names::transfer_mode, _T("default"));
 		break;
 	}
-	Bind(insertServerQuery_, server_table_column_names::max_connections, site.server.MaximumMultipleConnections());
+	Bind(insertServerQuery_, server_table_column_names::max_connections, site.connection_limit_);
 
 	switch (site.server.GetEncodingType())
 	{
@@ -1100,11 +1106,11 @@ int64_t CQueueStorage::Impl::ParseServerFromRow(Site & site)
 		site.server.SetPasvMode(MODE_DEFAULT);
 	}
 
-	int maximumMultipleConnections = GetColumnInt(selectServersQuery_, server_table_column_names::max_connections);
+	int64_t maximumMultipleConnections = GetColumnInt64(selectServersQuery_, server_table_column_names::max_connections);
 	if (maximumMultipleConnections < 0) {
 		return INVALID_DATA;
 	}
-	site.server.MaximumMultipleConnections(maximumMultipleConnections);
+	site.connection_limit_ = static_cast<unsigned int>(maximumMultipleConnections);
 
 	std::wstring encodingType = GetColumnText(selectServersQuery_, server_table_column_names::encoding);
 	if (encodingType.empty() || encodingType == _T("Auto")) {
