@@ -510,15 +510,15 @@ wxString CSiteManager::GetColourName(int i)
 	return wxGetTranslation(background_colors[i].name);
 }
 
-void CSiteManager::Rewrite(CLoginManager & loginManager, COptionsBase& options, pugi::xml_node element, bool on_failure_set_to_ask)
+void CSiteManager::Rewrite(CLoginManager & loginManager, COptionsBase& options, pugi::xml_node element, bool on_failure_set_to_ask, int64_t version)
 {
 	bool const forget = options.get_int(OPTION_DEFAULT_KIOSKMODE) != 0;
 	for (auto child = element.first_child(); child; child = child.next_sibling()) {
 		if (!strcmp(child.name(), "Folder")) {
-			Rewrite(loginManager, options, child, on_failure_set_to_ask);
+			Rewrite(loginManager, options, child, on_failure_set_to_ask, version);
 		}
 		else if (!strcmp(child.name(), "Server")) {
-			auto site = ReadServerElement(child);
+			auto site = ReadServerElement(child, version);
 			if (site) {
 				if (!forget) {
 					loginManager.AskDecryptor(site->credentials.encrypted_, true, false);
@@ -550,7 +550,7 @@ void CSiteManager::Rewrite(CLoginManager & loginManager, COptionsBase& options,
 		return;
 	}
 
-	Rewrite(loginManager, options, element, on_failure_set_to_ask);
+	Rewrite(loginManager, options, element, on_failure_set_to_ask, file.GetVersion());
 
 	SaveWithErrorDialog(file);
 }
@@ -607,14 +607,14 @@ bool CSiteManager::ImportSites(pugi::xml_node sites)
 		currentSites = element.append_child("Servers");
 	}
 
-	if (!ImportSites(sites, currentSites)) {
+	if (!ImportSites(sites, currentSites, file.GetVersion())) {
 		return false;
 	}
 
 	return SaveWithErrorDialog(file);
 }
 
-bool CSiteManager::ImportSites(pugi::xml_node sitesToImport, pugi::xml_node existingSites)
+bool CSiteManager::ImportSites(pugi::xml_node sitesToImport, pugi::xml_node existingSites, int64_t version)
 {
 	for (auto importFolder = sitesToImport.child("Folder"); importFolder; importFolder = importFolder.next_sibling("Folder")) {
 		std::wstring name = GetTextElement_Trimmed(importFolder, "Name").substr(0, 255);
@@ -632,13 +632,13 @@ bool CSiteManager::ImportSites(pugi::xml_node sitesToImport, pugi::xml_node exis
 			newName = fz::sprintf(L"%s %d", name.substr(0, 240), i++);
 		}
 
-		ImportSites(importFolder, folder);
+		ImportSites(importFolder, folder, version);
 	}
 
 	auto & loginManager = CLoginManager::Get();
 
 	for (auto importSite = sitesToImport.child("Server"); importSite; importSite = importSite.next_sibling("Server")) {
-		auto site = ReadServerElement(importSite);
+		auto site = ReadServerElement(importSite, version);
 		if (!site) {
 			continue;
 		}
