@@ -379,16 +379,17 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site>*
 	SetMinSize(GetSizer()->GetMinSize() + size - clientSize);
 	SetClientSize(minSize);
 
-	// Load bookmark notebook
-	m_pNotebook_Bookmark = new wxNotebook(this, -1);
+	{
+		// Load bookmark notebook
+		m_pNotebook_Bookmark = new wxNotebook(this, nullID);
 
-	auto * bookmarkPanel = CreateBookmarkPanel(m_pNotebook_Bookmark, lay);
-
-	m_pNotebook_Bookmark->Hide();
-	m_pNotebook_Bookmark->AddPage(bookmarkPanel, _("Bookmark"));
-	right->Add(m_pNotebook_Bookmark, 2, wxGROW);
-	right->SetItemMinSize(1, right->GetItem((size_t)0)->GetMinSize().GetWidth(), -1);
+		auto * bookmarkPanel = CreateBookmarkPanel(m_pNotebook_Bookmark, lay);
 
+		m_pNotebook_Bookmark->AddPage(bookmarkPanel, _("Bookmark"));
+		m_pNotebook_Bookmark->Hide();
+		right->Add(m_pNotebook_Bookmark, 2, wxGROW);
+		right->SetItemMinSize(1, right->GetItem((size_t)0)->GetMinSize().GetWidth(), -1);
+	}
 
 	// Set min size of tree to actual size of tree.
 	// Otherwise some platforms automatically calculate a min size fitting all items,
@@ -906,7 +907,7 @@ bool CSiteManagerDialog::Verify()
 
 	if (data->m_site) {
 		Site newSite = *data->m_site;
-		if (!m_pNotebook_Site->UpdateSite(newSite, false)) {
+		if (m_pNotebook_Site && !m_pNotebook_Site->UpdateSite(newSite, false)) {
 			return false;
 		}
 	}
@@ -1219,7 +1220,7 @@ bool CSiteManagerDialog::UpdateServer(Site & site, const wxString &name)
 {
 	Site newSite = site;
 	newSite.SetName(name.ToStdWstring());
-	if (!m_pNotebook_Site->UpdateSite(newSite, true)) {
+	if (m_pNotebook_Site && !m_pNotebook_Site->UpdateSite(newSite, true)) {
 		return false;
 	}
 
@@ -1315,11 +1316,15 @@ void CSiteManagerDialog::SetCtrlState()
 		// Set the control states according if it's possible to use the control
 		const bool root_or_predefined = (item == tree_->GetRootItem() || item == m_ownSites || predefined);
 
-		m_pNotebook_Site->Show();
-		m_pNotebook_Bookmark->Hide();
-		m_pNotebook_Site->SetSite(Site(), root_or_predefined || multiple);
-		m_pNotebook_Site->Enable(false);
-		m_pNotebook_Site->GetContainingSizer()->Layout();
+		if (m_pNotebook_Bookmark) {
+			m_pNotebook_Bookmark->Hide();
+		}
+		if (m_pNotebook_Site) {
+			m_pNotebook_Site->Show();
+			m_pNotebook_Site->SetSite(Site(), root_or_predefined || multiple);
+			m_pNotebook_Site->Enable(false);
+			m_pNotebook_Site->GetContainingSizer()->Layout();
+		}
 
 		xrc_call(*this, "ID_RENAME", &wxWindow::Enable, !selections.empty() && !root_or_predefined && !multiple);
 		xrc_call(*this, "ID_DELETE", &wxWindow::Enable, !selections.empty() && !root_or_predefined);
@@ -1333,11 +1338,15 @@ void CSiteManagerDialog::SetCtrlState()
 #endif
 	}
 	else if (data->m_site) {
-		m_pNotebook_Site->Show();
-		m_pNotebook_Bookmark->Hide();
-		m_pNotebook_Site->SetSite(*data->m_site, predefined);
-		m_pNotebook_Site->Enable(true);
-		m_pNotebook_Site->GetContainingSizer()->Layout();
+		if (m_pNotebook_Bookmark) {
+			m_pNotebook_Bookmark->Hide();
+		}
+		if (m_pNotebook_Site) {
+			m_pNotebook_Site->Show();
+			m_pNotebook_Site->SetSite(*data->m_site, predefined);
+			m_pNotebook_Site->Enable(true);
+			m_pNotebook_Site->GetContainingSizer()->Layout();
+		}
 
 		// Set the control states according if it's possible to use the control
 		xrc_call(*this, "ID_RENAME", &wxWindow::Enable, !predefined);
@@ -1353,9 +1362,13 @@ void CSiteManagerDialog::SetCtrlState()
 #endif
 	}
 	else {
-		m_pNotebook_Site->Hide();
-		m_pNotebook_Bookmark->Show();
-		m_pNotebook_Site->GetContainingSizer()->Layout();
+		if (m_pNotebook_Site) {
+			m_pNotebook_Site->Hide();
+			m_pNotebook_Site->GetContainingSizer()->Layout();
+		}
+		if (m_pNotebook_Bookmark) {
+			m_pNotebook_Bookmark->Show();
+		}
 
 		// Set the control states according if it's possible to use the control
 		xrc_call(*this, "ID_RENAME", &wxWindow::Enable, !predefined);
