@@ -36,7 +36,7 @@ bool CSiteManagerSite::Load(wxWindow* parent, bool with_comments_and_color)
 			wxWindowUpdateLocker l(this);
 			SetControlVisibility(protocol, type);
 			for (auto & controls : controls_) {
-				controls->SetControlState();
+				controls->SetControlState(predefined_);
 			}
 			GetContainingSizer()->Layout();
 		};
@@ -119,7 +119,9 @@ bool CSiteManagerSite::Load(wxWindow* parent, bool with_comments_and_color)
 
 	m_totalPages = GetPageCount();
 
-	GetPage(0)->GetSizer()->Fit(GetPage(0));
+	if (generalPage_) {
+		generalPage_->GetSizer()->Fit(generalPage_);
+	}
 
 #ifdef __WXMSW__
 	// Make pages at least wide enough to fit all tabs
@@ -149,7 +151,9 @@ bool CSiteManagerSite::Load(wxWindow* parent, bool with_comments_and_color)
 	}
 #endif
 
-	static_cast<GeneralSiteControls&>(*controls_[0]).UpdateWidth(width);
+	for (auto & c : controls_) {
+		c->UpdateWidth(width);
+	}
 
 	return true;
 }
@@ -157,7 +161,6 @@ bool CSiteManagerSite::Load(wxWindow* parent, bool with_comments_and_color)
 void CSiteManagerSite::SetControlVisibility(ServerProtocol protocol, LogonType type)
 {
 	for (auto & controls : controls_) {
-		controls->SetPredefined(predefined_);
 		controls->SetControlVisibility(protocol, type);
 	}
 
@@ -217,7 +220,9 @@ void CSiteManagerSite::SetControlVisibility(ServerProtocol protocol, LogonType t
 		}
 	}
 
-	GetPage(0)->GetSizer()->Fit(GetPage(0));
+	if (generalPage_) {
+		generalPage_->GetSizer()->Fit(generalPage_);
+	}
 }
 
 bool CSiteManagerSite::UpdateSite(Site &site, bool silent)
@@ -257,7 +262,7 @@ void CSiteManagerSite::SetSite(Site const& site, bool predefined)
 	}
 
 	for (auto & controls : controls_) {
-		controls->SetSite(site);
-		controls->SetControlState();
+		controls->SetSite(site, predefined_);
+		controls->SetControlState(predefined_);
 	}
 }
