@@ -13,7 +13,7 @@ class CSiteManagerSite : public wxNotebook
 public:
 	CSiteManagerSite(COptionsBase & options);
 
-	bool Load(wxWindow * parent);
+	bool Load(wxWindow * parent, bool with_comments_and_color = true);
 
 	bool UpdateSite(Site &site, bool silent);
 	void SetSite(Site const& site, bool predefined);
