@@ -4,13 +4,13 @@
 #if ENABLE_SFTP
 #include "sftp_crypt_info_dlg.h"
 #endif
-#include "sizeformatting.h"
 #include "speedlimits_dialog.h"
 #include "statusbar.h"
 #include "themeprovider.h"
 #include "verifycertdialog.h"
 
 #include "../include/activity_logger.h"
+#include "../include/sizeformatting.h"
 
 #include <libfilezilla/glue/wxinvoker.hpp>
 
@@ -402,7 +402,7 @@ void CStatusBar::DoDisplayQueueSize()
 	}
 
 	wxString queueSize = wxString::Format(_("Queue: %s%s"), m_hasUnknownFiles ? _T(">") : _T(""),
-		CSizeFormat::Format(m_size, true, m_sizeFormat, m_sizeFormatThousandsSep, m_sizeFormatDecimalPlaces));
+	    SizeFormatter::Format(m_size, true, m_sizeFormat, m_sizeFormatThousandsSep, m_sizeFormatDecimalPlaces));
 
 	SetStatusText(queueSize, FIELD_QUEUESIZE);
 }
@@ -509,12 +509,12 @@ void CStatusBar::UpdateSizeFormat()
 {
 	// 0 equals bytes, however just use IEC binary prefixes instead,
 	// exact byte counts for queue make no sense.
-	m_sizeFormat = CSizeFormat::_format(options_.get_int(OPTION_SIZE_FORMAT));
-	if (!m_sizeFormat) {
-		m_sizeFormat = CSizeFormat::iec;
+	m_sizeFormat = SizeFormat(options_.get_int(OPTION_SIZE_FORMAT));
+	if (m_sizeFormat == SizeFormat::bytes) {
+		m_sizeFormat = SizeFormat::iec;
 	}
 
-	m_sizeFormatThousandsSep = options_.get_int(OPTION_SIZE_USETHOUSANDSEP) != 0;
+	m_sizeFormatThousandsSep = options_.get_bool(OPTION_SIZE_USETHOUSANDSEP);
 	m_sizeFormatDecimalPlaces = options_.get_int(OPTION_SIZE_DECIMALPLACES);
 
 	MeasureQueueSizeWidth();
@@ -622,14 +622,14 @@ void CStatusBar::UpdateSpeedLimitsIcon()
 		tooltip = _("Speed limits are enabled, click to change.");
 		tooltip += _T("\n");
 		if (downloadLimit) {
-			tooltip += wxString::Format(_("Download limit: %s/s"), CSizeFormat::FormatUnit(downloadLimit, CSizeFormat::kilo));
+			tooltip += wxString::Format(_("Download limit: %s/s"), SizeFormatter(options_).Format(downloadLimit, UnitPrefix::kilo));
 		}
 		else {
 			tooltip += _("Download limit: none");
 		}
 		tooltip += _T("\n");
 		if (uploadLimit) {
-			tooltip += wxString::Format(_("Upload limit: %s/s"), CSizeFormat::FormatUnit(uploadLimit, CSizeFormat::kilo));
+			tooltip += wxString::Format(_("Upload limit: %s/s"), SizeFormatter(options_).Format(uploadLimit, UnitPrefix::kilo));
 		}
 		else {
 			tooltip += _("Upload limit: none");
@@ -736,18 +736,18 @@ void CStatusBar::UpdateActivityTooltip()
 	speeds[0] /= 2;
 	speeds[1] /= 2;
 
-	CSizeFormat::_format format = static_cast<CSizeFormat::_format>(options_.get_int(OPTION_SIZE_FORMAT));
-	if (format == CSizeFormat::bytes) {
-		format = CSizeFormat::iec;
+	auto format = SizeFormat(options_.get_int(OPTION_SIZE_FORMAT));
+	if (format == SizeFormat::bytes) {
+		format = SizeFormat::iec;
 	}
 
 
-	std::wstring const upSpeed = CSizeFormat::Format(speeds[0], true, format,
-													 options_.get_int(OPTION_SIZE_USETHOUSANDSEP) != 0,
+	std::wstring const upSpeed = SizeFormatter::Format(speeds[0], true, format,
+	                                                 options_.get_bool(OPTION_SIZE_USETHOUSANDSEP),
 													 options_.get_int(OPTION_SIZE_DECIMALPLACES));
 
-	std::wstring const dlSpeed = CSizeFormat::Format(speeds[1], true, format,
-													 options_.get_int(OPTION_SIZE_USETHOUSANDSEP) != 0,
+	std::wstring const dlSpeed = SizeFormatter::Format(speeds[1], true, format,
+	                                                 options_.get_bool(OPTION_SIZE_USETHOUSANDSEP),
 													 options_.get_int(OPTION_SIZE_DECIMALPLACES));
 
 
