@@ -415,6 +415,8 @@ int sftp_put_file(char *fname, char *outfname, int restart)
     bool err = false, eof;
     struct fxp_attrs attrs;
     long permissions;
+    _fztimer timer;
+    int rinterval;
 
     attrs.flags = 0;
 //FIXME    PUT_PERMISSIONS(attrs, permissions);
@@ -465,6 +467,9 @@ int sftp_put_file(char *fname, char *outfname, int restart)
     }
     fzprintf(sftpInfo, "local:%s => remote:%s\n", fname, outfname);
 
+    fz_timer_init(&timer);
+    rinterval = 0;
+
     /*
      * FIXME: we can use FXP_FSTAT here to get the file size, and
      * thus put up a progress bar.
@@ -476,6 +481,12 @@ int sftp_put_file(char *fname, char *outfname, int restart)
         int len, ret;
 
         while (xfer_upload_ready(xfer) && !err && !eof) {
+            if (rinterval) {
+                if (fz_timer_check(&timer)) {
+                    fzprintf(sftpTransfer, "%d", rinterval);
+                    rinterval = 0;
+                }
+            }
             len = read_from_file(file, buffer, sizeof(buffer));
             if (len == -1) {
                 fzprintf(sftpError, "error while reading local file");
@@ -484,6 +495,7 @@ int sftp_put_file(char *fname, char *outfname, int restart)
                 eof = true;
             } else {
                 xfer_upload_data(xfer, buffer, len);
+                rinterval += len;
                 if (pending_receive() >= 5)
                     break;
             }
@@ -513,6 +525,10 @@ int sftp_put_file(char *fname, char *outfname, int restart)
         }
     }
 
+    if (rinterval && !err) {
+        fzprintf(sftpTransfer, "%d", rinterval);
+    }
+
     xfer_cleanup(xfer);
 
   cleanup:
